@@ -166,12 +166,23 @@ sub get {
 	$path_info=~s/^/\//;
 	$path_info=~s/_/\//g;
     }
-    open(my $fh, '|-', "REMOTE_ADDR=CLEANUPREFRESH /usr/share/apt-cacher/apt-cacher -i -c $configfile >/dev/null") || die "Failed to open refresh pipe: $!";
-    printmsg "GET $path_info\n";
-    print $fh "GET $path_info\r\nCache-Control: max-age=0\r\nConnection: Close\r\n\r\n";
-    close($fh);
-    if($? && ! $force) {
-	die "Unable to update $path_info.\nCleanup aborted to prevent deletion of cached data.\n";
+    defined(my $refresh_pid = open(my $fh, '|-')) || die "Failed to open refresh pipe: $!";
+    if ($refresh_pid){
+	printmsg "Get $path_info\n";
+	print $fh "$_\r\n" foreach ("HEAD $path_info", 'Cache-Control: max-age=0', 'Connection: Close', '');
+	close($fh);	
+    }
+    else {
+	close (STDOUT);
+	open (STDOUT, '>', '/dev/null') || die $!;
+	local $ENV{REMOTE_ADDR} = 'CLEANUPREFRESH';
+	local @ARGV = ('-i', '-c', $configfile);
+	do '/usr/share/apt-cacher/apt-cacher';
+    }
+
+    waitpid($refresh_pid,0);
+    if(($? > 0) && ! $force) {
+	die "Unable to update $path_info (status: $?).\nCleanup aborted to prevent deletion of cached data.\n";
     }
     return $?;
 }
@@ -184,21 +195,27 @@ sub pdiff {
     }
 
     if ($name !~ /main|contrib|non-free/) {
-	printmsg "Upstream repository for $name not recognised layout, skipping attempting to patch\n";
+	printmsg "Upstream repository for $name not standard hierarchy, skipping attempting to patch\n";
 	return;
     }
     my ($basename,$type) = ($name =~ /(^.+?)(\.(?:bz2|gz))?$/);
-    (my $release = $basename) =~ s/(?:main|contrib|non-free).*$/Release/;
+    (my $release = $basename) =~ s/(?:main|contrib|non-free).*$/{In,}Release/;
     (my $diffindex = $basename) .= '.diff_Index';
 
-    for ($release, $diffindex) {
-	if (!-f $_ && !$offline) { # Refresh unless offline
-	    get($_);
-	}
-	if (!-f $_) {
-	    printmsg("$_ not available, aborting patch\n");
-	    return;
+    my ($release_fh, $diffin_fh);
+
+    foreach my $glob_fh ([\$release, \$release_fh], [\$diffindex, \$diffin_fh]) {
+	foreach (glob(${$glob_fh->[0]})) {
+	    get($_) unless $offline;
+	    open(${$glob_fh->[1]}, '<', $_)
+	      && do {
+		  ${$glob_fh->[0]} = $_;
+		  last;
+	      }
+		|| printmsg("Failed to open $_: $!\n");
 	}
+	return unless ${$glob_fh->[1]}->opened;
+	_flock(${$glob_fh->[1]}, LOCK_SH) || die("Cannot lock ${$glob_fh->[0]}: $!");
     }
 
     # Read Release file
@@ -208,11 +225,9 @@ sub pdiff {
 	s/_/\//g;
     }
 #    printmsg "Searching $release for $diffindex_patt and $name_patt\n";
-    open(my $rfh, '<', $release) || die "Unable to open $release: $!";
-    _flock($rfh, LOCK_SH);
 
     my ($diffindex_sha1, $name_sha1, $name_size);
-    while (<$rfh>) {
+    while (<$release_fh>) {
 	if (/^\s(\w{40})\s+\d+\s$diffindex_patt\n/) {
 	    $diffindex_sha1 = $1;
 #	    printmsg "Found! $diffindex_patt $1\n";
@@ -224,8 +239,8 @@ sub pdiff {
 	}
 	last if ($name_sha1 && $diffindex_sha1);
     }
-    _flock($rfh, LOCK_UN);
-    close($rfh);
+    _flock($release_fh, LOCK_UN);
+    close($release_fh);
     if (!$name_sha1 || !$name_size || !$diffindex_sha1) {
         warn "SHA1s for $name_patt and/or $diffindex_patt not found in $release, aborting patch\n";
 	return;
@@ -281,8 +296,6 @@ sub pdiff {
     # Read diff_Index
     my (@hist, @patch);
 
-    open(my $diffin_fh, '<', $diffindex) || die("Cannot open $diffindex: $!");
-    _flock($diffin_fh, LOCK_SH);
     my $diffindex_digest = $sha1->addfile($diffin_fh)->hexdigest;
     if ($diffindex_digest ne $diffindex_sha1) {
 	_flock($diffin_fh, LOCK_UN);
@@ -492,8 +505,12 @@ if (@db_mode || $db_recover){
     while (local $_ = shift @db_mode) {
 	/^import$/ && do {
 	    foreach (glob('*es.bz2 *es.gz *es *Release *diff_Index')) {
+		open(my $fh, '<', $_)|| do {
+		    warn "Failed to open $_ for import: $!";
+		    next;
+		};
 		printmsg "Importing checksums from $_\n";
-		import_sums($_) if !$sim_mode;
+		import_sums($_, $fh) if !$sim_mode;
 	    }
 	    next SWITCH;
 	};
@@ -608,7 +625,7 @@ if($> == 0 && !$cfg->{user} && !$force) {
 	}
 	foreach (glob) {
 	    if ( -f $_ ) {
-		print "Found $_\n";
+		printmsg "Found $_\n";
 		next CHECKFILE;
 	    }
 	}
@@ -617,37 +634,44 @@ if($> == 0 && !$cfg->{user} && !$force) {
     get($_) foreach keys %missing;
 }
 
-my @ifiles = glob('*{Release,diff_Index} *{Packages,Sources}{,.gz,.bz2}');
 
-foreach my $file (@ifiles) {
+# Initially preserve the index files
+%valid = map {$_ => 1} glob('*{Release,diff_Index} *{Packages,Sources}{,.gz,.bz2}');
 
-   # preserve the index files
-   $valid{$file}=1;
+foreach my $file (keys %valid) {
+
+    # Try to patch
+    my $patched;
+    if($pdiff_mode && $file =~ /(?:Packages|Sources)(?:\.(?:bz2|gz))?$/) {
+	printmsg "Attempting to update $file by patching\n";
+	($patched = pdiff($file)) || printmsg "Patching failed or not possible\n";
+    }
+    # If patching failed download them, unless offline
+    if (!$patched) {
+	if(!$offline) {
+	    get($file);
+	}
+	else {
+	    printmsg "Offline: Reusing existing $file\n";
+	}
+    }
+
+    # Remove obsolete Release and Release.gpg
+    if ((my $obsolete = $file) =~ s/InRelease/Release/) {
+	next unless exists $valid{$obsolete};
+	printmsg "Removing $obsolete in favour of $file\n";
+	delete $valid{$obsolete};
+	unlink $obsolete, "$obsolete.gpg" if !$sim_mode;
+    }
 
-   # Try to patch
-   my $patched;
-   if($pdiff_mode && $file =~ /(?:Packages|Sources)(?:\.(?:bz2|gz))?$/) {
-       printmsg "Attempting to update $file by patching\n";
-       ($patched = pdiff($file)) || printmsg "Patching failed or not possible\n";
-   }
-   # If patching failed download them, unless offline
-   if (!$patched) {
-       if(!$offline) {
-	   get($file);
-       }
-       else {
-	   printmsg "Offline: Reusing existing $file\n";
-       }
-   }
 }
 
-foreach my $file (@ifiles) {
+foreach my $file (keys %valid) {
+
     printmsg "Reading: $file\n";
 
     (my ($cfh, $hfh) = open_lock($file)) == 2 || do {
-	die "Failed to open filehandles for $file. Delete this manually or use --force if the repository is no longer interesting. \nExiting to prevent deletion of cache contents.\n" unless $force;
-	warn "Forced ignoring empty index file $file, apparently undownloadable. All packages referenced by it will be lost!\n";
-	next;
+	die "Failed to open filehandles for $file. Resolve this manually. \nExiting to prevent deletion of cache contents.\n";
     };
 
     extract_sums($file, $cfh, \%valid) || die("Error processing $file in $cfg->{cache_dir}/packages, cleanup stopped.\nRemove the file if the repository is no longer interesting and the packages pulled from it are to be removed.\n");
