@@ -54,7 +54,7 @@ sub read_config {
 		  request_timeout => 30,
 		  return_buffer_size => 1048576, # 1Mb
 		  reverse_path_map => 1,
-		  ubuntu_release_names => join(', ', qw(dapper edgy feisty gutsy hardy intrepid jaunty karmic lucid maverick natty oneiric precise)),
+		  ubuntu_release_names => join(', ', qw(dapper edgy feisty gutsy hardy intrepid jaunty karmic lucid maverick natty oneiric precise quantal)),
 		  use_proxy => 0,
 		  use_proxy_auth => 0,
 		  user => $>,
@@ -359,9 +359,13 @@ sub unlink_by_fh {
     my $count;
 
     foreach (@fh) {
-	next unless ref eq 'GLOB';
+	unless ((my $ref = ref) eq 'GLOB') {
+	    warn "Not a GLOB, skipping: $ref \n";
+	    next;
+	}
 	$_ =  filename_fh($_);
-	debug_message("Deleting $_") if defined &debug_message;
+	next unless -f; # Skip already deleted
+	debug_message("Deleting $_") if defined &debug_message && $cfg->{debug};
 	($count += unlink $_) || warn "Failed to delete $_: $!";
     }
     return $count;
@@ -383,10 +387,10 @@ sub _flock {
 	    debug_message('Waiting for '
 			  . join ('|', grep {$h{$_} & $flags && $_} keys %h)
 			  . ' on '
-			  . filename_fh($fh)) if defined &debug_message;
+			  . filename_fh($fh)) if defined &debug_message && $cfg->{debug};
 	}
         $ret = flock($fh, $flags);
-	debug_message('Got it!') if defined &debug_message;
+	debug_message('Got it!') if defined &debug_message && $cfg->{debug};
     }
     return $ret;
 }
@@ -433,7 +437,7 @@ sub read_header {
 sub write_header {
     my ($file, $response) = @_;
 
-    debug_message('Writing header') if defined (&debug_message);
+    debug_message('Writing header') if defined &debug_message && $cfg->{debug};
     # Remove Connection header and options
     foreach ($response->header('Connection')) {
 	$response->remove_header($_)
@@ -548,7 +552,7 @@ sub extract_sums {
        $namespace .= '/';
    }
 
-   my ($indexbase) = ($name =~ /([^\/]+_)(?:Index|Release)$/);
+   my ($indexbase) = ($name =~ /([^\/]+_)(?:Index|(?:In)?Release)$/);
    $indexbase = '' unless $indexbase; # Empty by default (for Sources)
 
    my ($skip,%data);
@@ -639,14 +643,11 @@ sub extract_sums {
 
 	$msg='Unspecified' if !$msg;
 
-	debug_message("Global lock: \u$msg") if defined (&debug_message);
+	debug_message("Global lock: \u$msg") if defined &debug_message && $cfg->{debug};
 
 	# May need to create it if the file got lost
-	sysopen($glock, $glockfile, O_CREAT|O_WRONLY) || die "Unable to open lockfile: $!";
-	$glock->autoflush;
+	sysopen($glock, $glockfile, O_CREAT) || die "Unable to open lockfile: $!";
 	_flock($glock, LOCK_EX) || die "Unable to lock $glockfile for \u$msg: $!";
-	$glock->truncate(0);
-	print $glock "$$: \u$msg\n";
 	return defined($glock);
     }
 
@@ -657,7 +658,7 @@ sub extract_sums {
 	}
 	_flock($glock, LOCK_UN) || die "Unable to release lock: $!";
 	close $glock || die "Unable to close lock: $!";
-	debug_message("Release global lock") if defined (&debug_message);
+	debug_message("Release global lock") if defined &debug_message &&  $cfg->{debug};
 	return;
     }
 
